/*******************************************************************************
 * Copyright Moxa Inc.
 *
 *
 * Date          Author            Comment
 * 09/11/2015    Daniel Chang      Created.
 ******************************************************************************/

 /*!
    \file CellularDataAccess.cpp
    \copyright Moxa Inc.
    \brief <b>\a Connect \a Device \a by \a CellularDataAccess</b>
    \date 09/11/2015
    \author Daniel Chang
    \version V1.0
    \details
*/

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <MXIO_V3_2500.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
//-----------------------------------------------------------------
#define TRUE    1
#define FALSE   0

#define HMODULE void
//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_2500, CMXIO_V3_2500 **pMXIO_V3_2500, char *szLibName);
int MXDestroyLibrary(HMODULE *hMXIO_V3_2500, CMXIO_V3_2500 **pMXIO_V3_2500);
char getch();
void WaitPressAnyKey();
void CheckErr(int ihConnection, int iRet, char *szFunctionName);

//-----------------------------------------------------------------
HMODULE *g_hMXIO_V3_2500 = NULL;
CMXIO_V3_2500 *g_pMXIO_V3_2500 = NULL;

//-----------------------------------------------------------------
//! \include CellularDataAccess.cpp
int main(int argc, char *argv[])
{
	char bytInputValue[64] = {'\0'};
	char szIP[] = "192.168.127.253";
	CMXIO_V3_2500::MX_MAC_ADDRESS MxMACAddr;
	memset(&MxMACAddr, 0, sizeof(MxMACAddr));
    char szMACAddr[18] = {'\0'};//"00-90-e8-00-00-41";
	DWORD dwTimeout = 3000;
	int ihConnection = 0;
	CMXIO_V3_2500::MX_ACCOUNT_DATA AccountData;
	sprintf(AccountData.szUserName,"%s", (char*)"admin");
	sprintf(AccountData.szPwd,"%s", (char*)"moxa");

	//-----------------------------------------------
	char IPAddress[16] = {'\0'};
	char szTempBuf[16] = {'\0'};
	WORD wPort = MXIO_V3_DEFAULT_CDA_PORT;
	sprintf(IPAddress, "%s", szIP);
	if (argc <= 1)
	{
		printf("Cmd Format: [IP Address] [Timeout] [TCP Port]\n");
		strcpy(IPAddress, szIP);
	}
	//
	if (argc > 1)
		sprintf(IPAddress, "%s", argv[1]);
	if (argc > 2)
		dwTimeout = atoi(argv[2]);
	if (argc > 3)
		wPort = atoi(argv[3]);
	printf("Input IP Address=%s, TimeOut=%d, TCP Port=%d\n", IPAddress, dwTimeout, wPort);
	//-----------------------
	int iRet = MXIO_V3_OK;
#ifdef _LOADX64MXIO
    iRet = MXCreateLibrary(g_hMXIO_V3_2500, &g_pMXIO_V3_2500, (char*)"/usr/local/lib/libmxio_2500_x64.so");
#else
    iRet = MXCreateLibrary(g_hMXIO_V3_2500, &g_pMXIO_V3_2500, (char*)"/usr/local/lib/libmxio_2500.so");
#endif
	printf("MXCreateLibrary() g_pMXIO_V3_2500=0x%p return %d(0x%X)\n", g_pMXIO_V3_2500, iRet, iRet);
	//-----------------------
	if (MXIO_V3_OK == iRet)
	{
		iRet = g_pMXIO_V3_2500->MXIO_Init();
		printf("MXIO_Init() return %d(0x%X)\n", iRet, iRet);
	}
	//-----------------------
	if (MXIO_V3_AGENT_PORT != wPort)
	{
		WORD wDevCount = MXIO_V3_OK;
		printf("Cellular Data Access IP = %s, Port=%d\n", IPAddress, wPort);
		iRet = g_pMXIO_V3_2500->MXIO_ListCDADevice(IPAddress, wPort, dwTimeout, &wDevCount);

		printf("MXIO_ListCDADevice(%d) return %d(0x%X)\n", ihConnection, iRet, iRet);
		//
		if (wDevCount)
		{
			CMXIO_V3_2500::MX_STRU_CDA_DEVICE_INFO *pCDADeviceInfo = new CMXIO_V3_2500::MX_STRU_CDA_DEVICE_INFO[wDevCount];
			iRet = g_pMXIO_V3_2500->MXIO_GetCDADeviceInfo(wDevCount, pCDADeviceInfo);
			printf("MXIO_GetCDADeviceInfo(%d) return %d(0x%X)\n", ihConnection, iRet, iRet);
			if (MXIO_V3_OK == iRet)
			{
				for (int i = 0; i < wDevCount; i++)
				{
					printf("========== Device %02d ==========\n", i);
					printf("ServerName=%s\n", pCDADeviceInfo[i].szServerName);
					printf("IP=%s\n", pCDADeviceInfo[i].szIP);
					printf("MAC = %02X-%02X-%02X-%02X-%02X-%02X",
                        pCDADeviceInfo[i].bytMAC[0],
                        pCDADeviceInfo[i].bytMAC[1],
                        pCDADeviceInfo[i].bytMAC[2],
                        pCDADeviceInfo[i].bytMAC[3],
                        pCDADeviceInfo[i].bytMAC[4],
                        pCDADeviceInfo[i].bytMAC[5]);

					printf("wUniqueID=%d, wModHWId=0x%X\n", pCDADeviceInfo[i].wUniqueID, pCDADeviceInfo[i].wModHWId);
				}
			}
			//Update Connect Device Info
			printf("Select CDA Device Index to connect:\n");
			int i = 0;
			char ch = 0;
			while((10 == (ch = getch())) && (i < (sizeof(bytInputValue)-1)))
			{
				bytInputValue[i] = ch;
				i++;
			}
			int iIndex = atoi(bytInputValue);
			if(iIndex >= wDevCount)
				iIndex = 0;
			sprintf(szMACAddr, "%02X-%02X-%02X-%02X-%02X-%02X",
				pCDADeviceInfo[iIndex].bytMAC[0],
				pCDADeviceInfo[iIndex].bytMAC[1],
				pCDADeviceInfo[iIndex].bytMAC[2],
				pCDADeviceInfo[iIndex].bytMAC[3],
				pCDADeviceInfo[iIndex].bytMAC[4],
				pCDADeviceInfo[iIndex].bytMAC[5]);
			printf("Connect CDA Device with MAC=%s\n", szMACAddr);
			memcpy(MxMACAddr.bytMAC, pCDADeviceInfo[iIndex].bytMAC, sizeof(BYTE)*6);
			//
			delete [] pCDADeviceInfo;
		}
	}
	//-----------------------
	if (MXIO_V3_OK == iRet)
	{
		sprintf(szMACAddr, "%02X-%02X-%02X-%02X-%02X-%02X",
				MxMACAddr.bytMAC[0],
				MxMACAddr.bytMAC[1],
				MxMACAddr.bytMAC[2],
				MxMACAddr.bytMAC[3],
				MxMACAddr.bytMAC[4],
				MxMACAddr.bytMAC[5]);
        printf("IP = %s, MAC = %s, Port=%d, Name=%s, Password=%s\n", 
			IPAddress, szMACAddr, wPort, AccountData.szUserName, AccountData.szPwd);		
        iRet = g_pMXIO_V3_2500->MXIO_Connect(IPAddress, wPort, dwTimeout, &MxMACAddr, &AccountData, &ihConnection);
		CheckErr(ihConnection, iRet, (char*)"MXIO_Connect");
	}
	//-----------------------------------------------------
	//
	//-----------------------------------------------------
	if (0 != ihConnection)
	{
		iRet = g_pMXIO_V3_2500->MXIO_Disconnect(ihConnection);
		CheckErr(ihConnection, iRet, (char*)"MXIO_Disconnect");
	}
	//-----------------------
	if (NULL != g_hMXIO_V3_2500)
	{
		iRet = g_pMXIO_V3_2500->MXIO_Exit();
		printf("MXIO_Exit() return %d(0x%X)\n", iRet, iRet);
		iRet = MXDestroyLibrary(g_hMXIO_V3_2500, &g_pMXIO_V3_2500);
		printf("MXDestroyLibrary() return %d(0x%X)\n", iRet, iRet);
	}
	WaitPressAnyKey();
	return 0;
}

//-----------------------------------------------------------------
int MXCreateLibrary(HMODULE *phMXIO_V3_2500, CMXIO_V3_2500 **pMXIO_V3_2500, char *szLibName)
{
	phMXIO_V3_2500 = dlopen(szLibName, RTLD_LAZY);
    if(!phMXIO_V3_2500)
    {
        printf("dlopen Error: %s\n", dlerror());
        return (-1);
    }
    PFNCREATEMXIO_V3_2500 pfnCreateMXIO_V3_2500 = (PFNCREATEMXIO_V3_2500)dlsym(phMXIO_V3_2500, "CreateMXIO_2500");
	if (NULL == pfnCreateMXIO_V3_2500)
	{
        printf("dlsym CreateMXIO Error: %s\n", dlerror());
        return (-1);
    }
	*pMXIO_V3_2500 = pfnCreateMXIO_V3_2500();
	return 0;
}
//-----------------------------------------------------------------
int MXDestroyLibrary(HMODULE *hMXIO_V3_2500, CMXIO_V3_2500 **pMXIO_V3_2500)
{
	PFNDESTROYMXIO_V3_2500 pfnDestroyMXIO_V3_2500 = (PFNDESTROYMXIO_V3_2500)dlsym(hMXIO_V3_2500, "DestroyMXIO_2500");
	if (!pfnDestroyMXIO_V3_2500)
	{
        printf("dlsym DestroyMXIO Error: %s\n", dlerror());
        return (-1);
    }

	pfnDestroyMXIO_V3_2500(*pMXIO_V3_2500);
	dlclose(hMXIO_V3_2500);
	*pMXIO_V3_2500 = NULL;
	return 0;
}
//-----------------------------------------------------------------
char getch()
{
    int iRet;
	char ch;
	struct termios oldt, newt;

	tcgetattr(STDIN_FILENO, &oldt);
	memcpy(&newt, &oldt, sizeof(newt));
	newt.c_lflag &= ~( ECHO | ICANON | ECHOE | ECHOK |
						ECHONL | ECHOPRT | ECHOKE | ICRNL);
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	iRet = read(STDIN_FILENO, &ch, 1);
	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	if(iRet == 1)
		return ch;
	else
		return 0;
}

void WaitPressAnyKey()
{
	printf("press any key to exit.\n");
	getch();
}

//-----------------------------------------------------------------
//===========================================================================
void CheckErr(int ihConnection, int iRet, char *szFunctionName)
{
	char *szErrMsg = (char*)"MXIO_V3_OK";
	if (iRet != MXIO_V3_OK)
	{
		switch (iRet)
		{
		case MXIO_V3_ILLEGAL_FUNCTION:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_FUNCTION";
			break;
		case MXIO_V3_ILLEGAL_DATA_ADDRESS:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_DATA_ADDRESS";
			break;
		case MXIO_V3_ILLEGAL_DATA_VALUE:
			szErrMsg = (char*)"MXIO_V3_ILLEGAL_DATA_VALUE";
			break;
		case MXIO_V3_SLAVE_DEVICE_FAILURE:
			szErrMsg = (char*)"MXIO_V3_SLAVE_DEVICE_FAILURE";
			break;
		case MXIO_V3_SLAVE_DEVICE_BUSY:
			szErrMsg = (char*)"MXIO_V3_SLAVE_DEVICE_BUSY";
			break;
		case MXIO_V3_EIO_TIME_OUT:
			szErrMsg = (char*)"MXIO_V3_EIO_TIME_OUT";
			break;
		case MXIO_V3_EIO_INIT_SOCKETS_FAIL:
			szErrMsg = (char*)"MXIO_V3_EIO_INIT_SOCKETS_FAIL";
			break;
		case MXIO_V3_EIO_CREATING_SOCKET_ERROR:
			szErrMsg = (char*)"MXIO_V3_EIO_CREATING_SOCKET_ERROR";
			break;
		case MXIO_V3_EIO_RESPONSE_BAD:
			szErrMsg = (char*)"MXIO_V3_EIO_RESPONSE_BAD";
			break;
		case MXIO_V3_EIO_SOCKET_DISCONNECT:
			szErrMsg = (char*)"MXIO_V3_EIO_SOCKET_DISCONNECT";
			break;
		case MXIO_V3_PROTOCOL_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_PROTOCOL_TYPE_ERROR";
			break;
		case MXIO_V3_PRODUCT_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_PRODUCT_NOT_SUPPORT";
			break;
		case MXIO_V3_HANDLE_ERROR:
		    szErrMsg = (char*)"MXIO_V3_HANDLE_ERROR";
		    break;
		case MXIO_V3_SLOT_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_SLOT_OUT_OF_RANGE";
			break;
		case MXIO_V3_CHANNEL_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_CHANNEL_OUT_OF_RANGE";
			break;
		case MXIO_V3_COIL_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_COIL_TYPE_ERROR";
			break;
		case MXIO_V3_REGISTER_TYPE_ERROR:
			szErrMsg = (char*)"MXIO_V3_REGISTER_TYPE_ERROR";
			break;
		case MXIO_V3_FUNCTION_NOT_SUPPORT:
		    szErrMsg = (char*)"MXIO_V3_FUNCTION_NOT_SUPPORT";
		    break;
		case MXIO_V3_OUTPUT_VALUE_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_OUTPUT_VALUE_OUT_OF_RANGE";
			break;
		case MXIO_V3_INPUT_VALUE_OUT_OF_RANGE:
			szErrMsg = (char*)"MXIO_V3_INPUT_VALUE_OUT_OF_RANGE";
			break;
		case MXIO_V3_EIO_PASSWORD_INCORRECT:
			szErrMsg = (char*)"MXIO_V3_EIO_PASSWORD_INCORRECT";
			break;
		case MXIO_V3_SLOT_NOT_EXIST:
			szErrMsg = (char*)"MXIO_V3_SLOT_NOT_EXIST";
			break;
		case MXIO_V3_FIRMWARE_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_FIRMWARE_NOT_SUPPORT";
			break;
		case MXIO_V3_CREATE_MUTEX_FAIL:
			szErrMsg = (char*)"MXIO_V3_CREATE_MUTEX_FAIL";
			break;
		case MXIO_V3_ERR_ENUM_NET_INTERFACE_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_ENUM_NET_INTERFACE_FAIL";
			break;
		case MXIO_V3_ERR_SELECT_NET_INTERFACE_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_SELECT_NET_INTERFACE_FAIL";
			break;
		case MXIO_V3_ERR_FUNCTION_NOT_SUPPORT:
			szErrMsg = (char*)"MXIO_V3_ERR_FUNCTION_NOT_SUPPORT";
			break;
		case MXIO_V3_ERR_INVALID_HANDLE:
			szErrMsg = (char*)"MXIO_V3_ERR_INVALID_HANDLE";
			break;
		case MXIO_V3_ERR_BLOCK_QUANTITY:
			szErrMsg = (char*)"MXIO_V3_ERR_BLOCK_QUANTITY";
			break;
		case MXIO_V3_ERR_TOTAL_BLOCK_SIZE:
			szErrMsg = (char*)"MXIO_V3_ERR_TOTAL_BLOCK_SIZE";
			break;
		case MXIO_V3_ERR_REACH_MXIO_LIMIT_CONNECTION:
			szErrMsg = (char*)"MXIO_V3_ERR_REACH_MXIO_LIMIT_CONNECTION";
			break;
		case MXIO_V3_ERR_PARSE_XML_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_PARSE_XML_FAIL";
			break;
		case MXIO_V3_ERR_ACCOUNT_VEIFY_FAIL:
			szErrMsg = (char*)"MXIO_V3_ERR_ACCOUNT_VEIFY_FAIL";
			break;
		case MXIO_V3_ERR_ACCOUNT_ACCESS_RIGHT:
			szErrMsg = (char*)"MXIO_V3_ERR_ACCOUNT_ACCESS_RIGHT";
			break;
		default:
			szErrMsg = (char*)"Unknown Error Value";
			break;
		}
		printf("%d Function \"%s\" execution Fail.\nError Message : %s (%d(0x%X))\n",
			   ihConnection, szFunctionName, szErrMsg, iRet, iRet);
	}
	else
	{
		printf("%d Function \"%s\" execution Success\n", ihConnection, szFunctionName);
	}
}
